@@ -434,6 +434,7 @@ class __ccp_re__(object):
 
 
 def _get_ipv4(val="", strict=False, stdlib=False, debug=0):
+    """Return the requested IPv4 object to the caller.  This method heavily depends on IPv4Obj()"""
     assert isinstance(val, str) or isinstance(val, int)
     assert isinstance(strict, bool)
     assert isinstance(stdlib, bool)
@@ -461,6 +462,7 @@ def _get_ipv4(val="", strict=False, stdlib=False, debug=0):
 
 
 def _get_ipv6(val="", strict=False, stdlib=False, debug=0):
+    """Return the requested IPv6 object to the caller.  This method heavily depends on IPv6Obj()"""
     assert isinstance(val, str) or isinstance(val, int)
     assert isinstance(strict, bool)
     assert isinstance(stdlib, bool)
@@ -508,14 +510,13 @@ def ip_factory(val="", stdlib=False, mode="auto_detect", debug=0):
         if isinstance(val, str) and (":" in val):
             obj = _get_ipv6(val=val, stdlib=stdlib, debug=debug)
 
-        elif isinstance(val, str):
+        elif isinstance(val, str) and not (":" in val):
             obj = _get_ipv4(val=val, stdlib=stdlib, debug=debug)
 
         elif isinstance(val, int):
-            if val <= IPV4_MAXINT:
-                obj = _get_ipv4(val=val, stdlib=stdlib, debug=debug)
-            else:
-                obj = _get_ipv6(val=val, stdlib=stdlib, debug=debug)
+            # Do not try to make ip version assumptions for integer inputs...
+            error_msg = "ip_factory(val=%s, mode='auto_detect') does not support integer inputs" % val
+            raise NotImplementedError(error_msg)
 
         if obj is not None:
             return obj
@@ -580,7 +581,8 @@ def collapse_addresses(network_list):
     return ipaddress.collapse_addresses([ip_net(ii) for ii in network_list])
 
 
-# Build a wrapper around ipaddress classes so we can customize behavoir
+# Build a wrapper around ipaddress classes so we can customize
+# behavior (like persisting host-bits when the masklen changes) and custom @properties
 class IPv4Obj(object):
     def __init__(self, arg="127.0.0.1/{0}".format(IPV4_MAX_PREFIXLEN), strict=False, debug=0):
         """An object to represent IPv4 addresses and IPv4 networks.
@@ -698,19 +700,36 @@ class IPv4Obj(object):
         version : int
             Returns an integer representing the IP version of this object.  Only 4 or 6 are valid results
         """
+        if debug > 0:
+            logger.info("IPv4Obj(arg='%s', strict=%s, debug=%s) was called" % (arg, strict, debug))
 
         self.arg = arg
         self.dna = "IPv4Obj"
-        # self.arg_ip, self.arg_mask = re.split(r"[\s\/]\s*", arg, maxsplit=2)
-        # self.ip_object = IPv4Address(self.arg_ip)
-        # self.network_object = IPv4Network("{0}/{1}".format(self.arg_ip, self.arg_mask), strict=False)
         self.ip_object = None
         self.network_object = None
         self.strict = strict
+        self.debug = debug
+        self.params_dict = {}
 
+        # Build params_dict... this needs to work with any supported input...
+        if isinstance(arg, str) or isinstance(arg, int):
+            params_dict = self._ipv4_params_dict(arg)
+            self.params_dict = params_dict
+
+        elif isinstance(arg, IPv4Obj):
+            params_dict = {
+                'ipv4_addr': str(arg.ip),
+                'ip_version': 4,
+                'ip_arg_str': str(arg.ip)+"/"+str(arg.masklen),
+                'netmask': str(arg.netmask),
+                'masklen': str(arg.masklen),
+            }
+            self.params_dict = params_dict
+
+        else:
+            raise ValueError("type(%s) is not supported" % arg)
 
         if isinstance(arg, str):
-            params_dict = self._ipv4_params_dict(arg)
             # Removing string length checks in 1.6.29... there are too many
             #    options such as IPv4Obj("111.111.111.111      255.255.255.255")
             # RECURSION
@@ -721,16 +740,8 @@ class IPv4Obj(object):
 
         elif isinstance(arg, int):
             assert 0 <= arg <= IPV4_MAXINT
-            #obj = _get_ipv4(arg, strict=strict)
-            #self.network_object, self.ip_object = obj.network, obj.ip
-
             self.network_object = IPv4Network(arg, strict=strict)
             self.ip_object = IPv4Address(arg)
-
-            #self.ip_object = IPv4Address(arg)
-            #self.network_object = IPv4Network(
-            #    str(self.ip_object) + "/" + str(IPV4_MAX_PREFIXLEN), strict=False
-            #)
             return None
 
         elif isinstance(arg, IPv4Obj):
@@ -750,15 +761,16 @@ class IPv4Obj(object):
             return None
 
         else:
-            raise ValueError(
+            raise ipaddress.AddressValueError(
                 "Could not parse '{0}' (type: {1}) into an IPv4 Address".format(
                     arg, type(arg)
                 )
             )
 
     # On IPv4Obj()
-    def _ipv4_params_dict(self, arg):
-        assert isinstance(arg, str)
+    def _ipv4_params_dict(self, arg, debug=0):
+        """Parse out important IPv4 parameters from arg.  This method must run to completion for address parsing to work correctly."""
+        assert isinstance(arg, str) or isinstance(arg, int) or isinstance(arg, IPv4Obj)
 
         params_dict = {
             'ipv4_addr': None,
@@ -767,51 +779,64 @@ class IPv4Obj(object):
             'netmask': None,
             'masklen': None,
         }
-        try:
-            mm = _RGX_IPV4ADDR_NETMASK.search(arg)
-        except TypeError:
-            raise ipaddress.AddressValueError(
-                "_ipv4_params_dict() doesn't understand how to parse {0}".format(arg)
+
+        if isinstance(arg, str):
+            try:
+                mm = _RGX_IPV4ADDR_NETMASK.search(arg)
+            except TypeError:
+                raise ipaddress.AddressValueError(
+                    "_ipv4_params_dict() doesn't understand how to parse {0}".format(arg)
+                )
+
+            error_msg = "_ipv4_params_dict() couldn't parse '{0}'".format(arg)
+            assert mm is not None, error_msg
+
+            mm_result = mm.groupdict()
+            addr = (
+                mm_result["addr0"]
+                or mm_result["addr1"]
+                or mm_result["addr2"]
+                or "127.0.0.1"
             )
+            ## Normalize if we get zero-padded strings, i.e. 172.001.001.001
+            assert re.search(r"^\d+\.\d+.\d+\.\d+", addr)
+            addr = ".".join([str(int(ii)) for ii in addr.split(".")])
 
-        ERROR = "_ipv4_params_dict() couldn't parse '{0}'".format(arg)
-        assert mm is not None, ERROR
+            netmask = mm_result["netmask"]
 
-        mm_result = mm.groupdict()
-        addr = (
-            mm_result["addr0"]
-            or mm_result["addr1"]
-            or mm_result["addr2"]
-            or "127.0.0.1"
-        )
+            masklen = int(mm_result.get("masklen", None) or IPV4_MAX_PREFIXLEN)
 
-        ## Normalize addr if we get zero-padded strings, i.e. 172.001.001.001
-        assert re.search(r"^\d+\.\d+.\d+\.\d+", addr)
-        addr = ".".join([str(int(ii)) for ii in addr.split(".")])
+            if netmask is not None:
+                ip_arg_str = "{0}/{1}".format(addr, netmask)
+            elif masklen is not None:
+                ip_arg_str = "{0}/{1}".format(addr, masklen)
+            else:
+                raise ipaddress.AddressValueError()
 
-        netmask = mm_result["netmask"]
+        elif isinstance(arg, int):
+            addr = str(IPv4Address(arg))
+            netmask = "255.255.255.255"
+            masklen = 32
+            ip_arg_str = "{0}/{1}".format(addr, masklen)
+
+        elif isinstance(arg, IPv4Obj):
+            addr = str(arg.ip)
+            netmask = str(arg.netmask)
+            masklen = arg.masklen
+            ip_arg_str = "{0}/{1}".format(addr, masklen)
+
+        else:
+            raise ipaddress.AddressValueError("IPv4Obj(arg='%s') is an unknown argument type" % (arg))
 
-        masklen = int(mm_result.get("masklen", None) or IPV4_MAX_PREFIXLEN)
         assert 0 <= masklen <= IPV4_MAX_PREFIXLEN
-
-
         params_dict = {
             'ipv4_addr': addr,
             'ip_version': 4,
-            'ip_arg_str': None,
+            'ip_arg_str': ip_arg_str,
             'netmask': netmask,
             'masklen': masklen,
         }
 
-        if params_dict.get('netmask', None) is not None:
-            ip_arg_str = "{0}/{1}".format(params_dict['ipv4_addr'], params_dict['netmask'])
-        elif params_dict.get('masklen', None) is not None:
-            ip_arg_str = "{0}/{1}".format(params_dict['ipv4_addr'], params_dict['masklen'])
-        else:
-            raise ipaddress.AddressValueError()
-
-        params_dict['ip_arg_str'] = ip_arg_str
-
         return params_dict
 
     # On IPv4Obj()
@@ -1069,6 +1094,11 @@ class IPv4Obj(object):
             "{0}/{1}".format(str(self.ip_object), arg), strict=False
         )
 
+    # On IPv4Obj()
+    @property
+    def netmask(self):
+        return self.network_object.netmask
+
     # On IPv4Obj()
     @property
     def masklength(self):
@@ -1276,7 +1306,8 @@ class IPv4Obj(object):
         return self.network_object.is_reserved
 
 
-# Build a wrapper around ipaddress classes so we can customize behavoir
+# Build a wrapper around ipaddress classes so we can customize
+# behavior (like persisting host-bits when the masklen changes) and custom @properties
 class IPv6Obj(object):
     def __init__(self, arg="::1/{0}".format(IPV6_MAX_PREFIXLEN), strict=False, debug=0):
         """An object to represent IPv6 addresses and IPv6 networks.
@@ -1336,36 +1367,32 @@ class IPv6Obj(object):
 
         """
 
+        if debug > 0:
+            logger.info("IPv6Obj(arg='%s', strict=%s, debug=%s) was called" % (arg, strict, debug))
+
         self.arg = arg
         self.dna = "IPv6Obj"
         self.ip_object = None
         self.network_object = None
         self.strict = strict
+        self.debug = debug
+        self.params_dict = {}
+
+        # Build params_dict... this needs to work with any supported input...
+        if isinstance(arg, str) or isinstance(arg, int) or isinstance(arg, IPv6Obj):
+            params_dict = self._ipv6_params_dict(arg)
+            self.params_dict = params_dict
 
         if isinstance(arg, str):
             assert len(arg) <= IPV6_MAXSTR_LEN
-            params_dict = self._ipv6_params_dict(arg)
-            #network_obj, ip_obj = self._parse_ipv6obj_from_string(arg)
-            #self.network_object, self.ip_object = network_obj, ip_obj
             self.network_object = IPv6Network(params_dict['ip_arg_str'], strict=strict)
             self.ip_object = IPv6Address(params_dict['ipv6_addr'])
-
-            #obj = _get_ipv6(arg, strict=strict)
-            #self.network_object, self.ip_object = obj.network, obj.ip
             return None
 
         elif isinstance(arg, int):
             assert 0 <= arg <= IPV6_MAXINT
-            #obj = _get_ipv6(arg, strict=strict)
-            #self.network_object, self.ip_object = obj.network, obj.ip
-
             self.network_object = IPv6Network(arg, strict=strict)
             self.ip_object = IPv6Address(arg)
-
-            #self.ip_object = IPv6Address(arg)
-            #self.network_object = IPv6Network(
-            #    str(self.ip_object) + "/" + str(IPV6_MAX_PREFIXLEN), strict=False
-            #)
             return None
 
         elif isinstance(arg, IPv6Obj):
@@ -1391,11 +1418,10 @@ class IPv6Obj(object):
                 )
             )
 
-        # arg= _RGX_IPV6ADDR_NETMASK.sub(r'\1/\2', arg) # mangle IOS: 'addr mask'
-
     # On IPv6Obj()
-    def _ipv6_params_dict(self, arg):
-        assert isinstance(arg, str)
+    def _ipv6_params_dict(self, arg, debug=0):
+        """Parse out important IPv6 parameters from arg.  This method must run to completion for address parsing to work correctly."""
+        assert isinstance(arg, str) or isinstance(arg, int) or isinstance(arg, IPv6Obj)
 
         params_dict = {
             'ipv6_addr': None,
@@ -1404,43 +1430,62 @@ class IPv6Obj(object):
             'netmask': None,
             'masklen': None,
         }
-        try:
-            #mm = _RGX_IPV6ADDR_NETMASK.search(self.arg)
-            mm = _RGX_IPV6ADDR.search(arg)
 
-        except TypeError:
-            raise ipaddress.AddressValueError(
-                "_ipv6_params_dict() doesn't know how to parse {0}".format(arg)
-            )
+        if isinstance(arg, str):
+            try:
+                mm = _RGX_IPV6ADDR.search(arg)
+
+            except TypeError:
+                raise ipaddress.AddressValueError(
+                    "_ipv6_params_dict() doesn't know how to parse {0}".format(arg)
+                )
 
 
-        ERROR = "_ipv6_params_dict() couldn't parse '{0}'".format(arg)
-        assert mm is not None, ERROR
+            ERROR = "_ipv6_params_dict() couldn't parse '{0}'".format(arg)
+            assert mm is not None, ERROR
 
-        mm_result = mm.groupdict()
-        try:
-            addr = mm_result["addr"]
+            mm_result = mm.groupdict()
+            try:
+                addr = mm_result["addr"]
 
-        except Exception as ee:
-            addr = "::1"
+            except Exception as ee:
+                addr = "::1"
 
-        try:
-            masklen = int(mm_result['masklen'])
-        except Exception as ee:
-            masklen = IPV6_MAX_PREFIXLEN
+            try:
+                masklen = int(mm_result['masklen'])
+            except Exception as ee:
+                masklen = IPV6_MAX_PREFIXLEN
+
+            netmask_int = 2**(128 - masklen) - 1
+            netmask = str(IPv6Address(netmask_int))
+
+        elif isinstance(arg, int):
+            # Assume this arg int() represents an IPv6 host-address
+            addr = str(IPv6Address(arg))
+            netmask = 'ffff:ffff:ffff:ffff:ffff:ffff:ffff:ffff'
+            masklen = 128
+            ip_arg_str = "{0}/{1}".format(addr, masklen)
+
+        elif isinstance(arg, IPv6Obj):
+            addr = str(arg.ip)
+            netmask = str(arg.netmask)
+            masklen = arg.masklen
+            ip_arg_str = "{0}/{1}".format(addr, masklen)
+
+        else:
+            raise ipaddress.AddressValueError("IPv6Obj(arg='%s') is an unknown argument type" % (arg))
 
         assert 0 <= masklen <= IPV6_MAX_PREFIXLEN
-
         params_dict = {
             'ipv6_addr': addr,
             'ip_version': 6,
             'ip_arg_str': None,
-            'netmask': None,
+            'netmask': netmask,
             'masklen': masklen,
         }
 
         if params_dict.get('masklen', None) is not None:
-            ip_arg_str = "{0}/{1}".format(params_dict['ipv6_addr'], params_dict['masklen'])
+            ip_arg_str = "{0}/{1}".format(addr, masklen)
         else:
             raise ipaddress.AddressValueError()
 
@@ -2111,14 +2156,14 @@ def dns_query(input_str="", query_type="", server="", timeout=2.0):
     assert input_str != ""
     # input = input_str.strip()
     retval = set([])
-    resolver = Resolver()
-    resolver.server = [socket.gethostbyname(server)]
-    resolver.timeout = float(timeout)
-    resolver.lifetime = float(timeout)
+    rr = Resolver()
+    rr.server = [socket.gethostbyname(server)]
+    rr.timeout = float(timeout)
+    rr.lifetime = float(timeout)
     start = time.time()
     if (query_type == "A") or (query_type == "AAAA"):
         try:
-            answer = resolver.query(input_str, query_type)
+            answer = rr.query(input_str, query_type)
             duration = time.time() - start
             for result in answer:
                 response = DNSResponse(
@@ -2142,7 +2187,7 @@ def dns_query(input_str="", query_type="", server="", timeout=2.0):
         return [_zone[node].to_text(node) for node in _zone.nodes.keys()]
     elif query_type == "CNAME":
         try:
-            answer = resolver.query(input_str, query_type)
+            answer = rr.query(input_str, query_type)
             duration = time.time() - start
             for result in answer:
                 response = DNSResponse(
@@ -2162,7 +2207,7 @@ def dns_query(input_str="", query_type="", server="", timeout=2.0):
             retval.add(response)
     elif query_type == "MX":
         try:
-            answer = resolver.query(input_str, query_type)
+            answer = rr.query(input_str, query_type)
             duration = time.time() - start
             for result in answer:
                 response = DNSResponse(
@@ -2182,7 +2227,7 @@ def dns_query(input_str="", query_type="", server="", timeout=2.0):
             retval.add(response)
     elif query_type == "NS":
         try:
-            answer = resolver.query(input_str, query_type)
+            answer = rr.query(input_str, query_type)
             duration = time.time() - start
             for result in answer:
                 response = DNSResponse(
@@ -2222,7 +2267,7 @@ def dns_query(input_str="", query_type="", server="", timeout=2.0):
             raise ValueError('Cannot query PTR record for "{0}"'.format(input_str))
 
         try:
-            answer = resolver.query(inaddr, query_type)
+            answer = rr.query(inaddr, query_type)
             duration = time.time() - start
             for result in answer:
                 response = DNSResponse(
@@ -2242,7 +2287,7 @@ def dns_query(input_str="", query_type="", server="", timeout=2.0):
             retval.add(response)
     elif query_type == "TXT":
         try:
-            answer = resolver.query(input_str, query_type)
+            answer = rr.query(input_str, query_type)
             duration = time.time() - start
             for result in answer:
                 response = DNSResponse(
@@ -2263,22 +2308,39 @@ def dns_query(input_str="", query_type="", server="", timeout=2.0):
     return retval
 
 
-def dns_lookup(input_str, timeout=3, server=""):
+def dns_lookup(input_str, timeout=3, server="", record_type="A"):
     """Perform a simple DNS lookup, return results in a dictionary"""
-    resolver = Resolver()
-    resolver.timeout = float(timeout)
-    resolver.lifetime = float(timeout)
+    rr = Resolver()
+    rr.timeout = float(timeout)
+    rr.lifetime = float(timeout)
     if server:
-        resolver.nameservers = [server]
+        rr.nameservers = [server]
+    dns_session = rr.resolve(input_str, record_type)
+    responses = list()
+    for rdata in dns_session:
+        responses.append(str(rdata))
+
+    """
+    from dns import resolver
+    rr = resolver.Resolver()
+    rr.nameservers = ['8.8.8.8']
+    rr.timeout = 2.0
+    foo = rr.resolve('cisco.com', 'A')
+    for rdata in foo:
+        print("ADDR", rdata)
+    """
+
     try:
-        records = resolver.query(input_str, "A")
         return {
-            "addrs": [ii.address for ii in records],
+            #"addrs": [ii.address for ii in records],
+            "record_type": record_type,
+            "addrs": responses,
             "error": "",
             "name": input_str,
         }
     except DNSException as e:
         return {
+            "record_type": record_type,
             "addrs": [],
             "error": repr(e),
             "name": input_str,
@@ -2287,13 +2349,13 @@ def dns_lookup(input_str, timeout=3, server=""):
 
 def dns6_lookup(input_str, timeout=3, server=""):
     """Perform a simple DNS lookup, return results in a dictionary"""
-    resolver = Resolver()
-    resolver.timeout = float(timeout)
-    resolver.lifetime = float(timeout)
+    rr = Resolver()
+    rr.timeout = float(timeout)
+    rr.lifetime = float(timeout)
     if server:
-        resolver.nameservers = [server]
+        rr.nameservers = [server]
     try:
-        records = resolver.query(input_str, "AAAA")
+        records = rr.query(input_str, "AAAA")
         return {
             "addrs": [ii.address for ii in records],
             "error": "",
@@ -2315,16 +2377,16 @@ def reverse_dns_lookup(input_str, timeout=3, server=""):
     assert _REVERSE_DNS_REGEX.search(input_str), "Invalid address format: '{0}'".format(
         input_str
     )
-    resolver = Resolver()
-    resolver.timeout = float(timeout)
-    resolver.lifetime = float(timeout)
+    rr = Resolver()
+    rr.timeout = float(timeout)
+    rr.lifetime = float(timeout)
     if server:
-        resolver.nameservers = [server]
+        rr.nameservers = [server]
     try:
         tmp = input_str.strip().split(".")
         tmp.reverse()
         inaddr = ".".join(tmp) + ".in-addr.arpa"
-        records = resolver.query(inaddr, "PTR")
+        records = rr.query(inaddr, "PTR")
         return {
             "name": records[0].to_text(),
             "lookup": inaddr,
